program MacaCliente;
{$APPTYPE CONSOLE}

uses SysUtils, Classes, Corba, COSNaming, NMHttp, Psock, Dialogs, StrUtils,
     Security_c, Security_i, SecurityLevel2_c, SecurityLevel2_i,
     DfResourceAccessDecision_c, DfResourceAccessDecision_i,
     maca_cs_c, maca_cs_i, maca_cs_s;

type
  //Classe que implementa o mtodo de callback "isClientAlive"
  //no cliente. Este  o nico mtodo existente na interface
  //"Callback" e deve ser implementado com a extenso da
  //classe "TInterfacedObject" do Delphi
  TCallback = class(TInterfacedObject, Callback)
  public
    constructor Create;
    //Mtodo de callback implementado pelo cliente e
    //invocado pelo servidor MACA CS. Embora seja chamado
    //com modo "oneway", sua execuo deve ser a mais breve
    //possvel
    procedure isClientAlive (const message : AnsiString);
  end;

  constructor TCallback.Create;
  begin
    inherited;
  end;

  procedure TCallback.isClientAlive (const message : AnsiString);
  begin
    Writeln(message);
  end;

Function autorizaMultiplosAcessos (creds : Credentials; ado : AccessDecision;
                                   nomeRecursos, operacoes : TStrings) : BooleanList;
var
  extFamily       : ExtensibleFamily;
  attrTypeRole,
  attrTypeID      : AttributeType;
  attrs           : Security_i.AttributeList;
  attrTypeList    : AttributeTypeList;
  access_requests : DfResourceAccessDecision_i.AccessDefinitionList;
  resNameCompts   : ResourceNameComponentList;
  resName         : ResourceName;
  res             : BooleanList;
  n               : Integer;
begin
  SetLength(attrs, 0);
  SetLength(res, 0);
  result := res;
  //Verifica se a credencial e 'ado' foram criados
  if ((creds <> nil) and (ado <> nil) and (nomeRecursos.Count = operacoes.Count)) then begin
    try
      //Define a famlia dos atributos de segurana
      extFamily    := TExtensibleFamily.Create(0, 1);
      //Define os tipos de atributos a recuperar, Role e AccessId neste caso
      attrTypeRole := TAttributeType.Create(extFamily, Role);
      SetLength(attrTypeList, 2);
      attrTypeList[0] := attrTypeRole;
      attrTypeID      := TAttributeType.Create(extFamily, AccessId);
      attrTypeList[1] := attrTypeID;
      //Recupera os atributos de segurana das credenciais do usurio
      attrs := creds.get_attributes(attrTypeList);
      //Define a lista de requisies de autorizao de acesso
      SetLength(access_requests, nomeRecursos.Count);
      //Para cada recurso, monta o componente com o nome do recurso e operao, e o inclui
      //na lista 'access_requests'
      for n := 0 to nomeRecursos.Count - 1 do begin
          SetLength(resNameCompts, 1);
          resNameCompts[0]   := TResourceNameComponent.Create('Aplicacoes', nomeRecursos[n]);
          resName            := TResourceName.Create('127.0.0.1', resNameCompts);
          access_requests[n] := TAccessDefinition.Create(resName, operacoes[n]);
      end;
      //Invoca o mtodo que retorna um array de booleanos, cada booleano correspondendo ao
      //resultado da solicitaao de acesso ao recurso correspondente a mesma posio do array
      result := ado.multiple_access_allowed(access_requests, attrs);
    except
      //O mtodo 'multiple_access_allowed' pode levantar esta exceo quando da ocorrncia de erros
      on exc : EInternalError do
        Writeln(exc.Message);
    end;
  end;
  //Libera memria dos arrays dinmicos
  access_requests := nil;
  res             := nil;
  attrTypeList    := nil;
  attrs           := nil;
end;

Function acessoAutorizado (creds : Credentials; ado : AccessDecision;
                           nomeRecurso, operacao : String) : boolean;
var
  extFamily     : ExtensibleFamily;
  attrTypeRole,
  attrTypeID    : AttributeType;
  attrs         : Security_i.AttributeList;
  attrTypeList  : AttributeTypeList;
  resNameCompts : ResourceNameComponentList;
  resNameCompt  : ResourceName;
begin
  SetLength(attrs, 0);
  result := false;
  //Verifica se a credencial e 'ado' foram criados
  if ((creds <> nil) and (ado <> nil)) then begin
    //Define a famlia dos atributos de segurana
    extFamily    := TExtensibleFamily.Create(0, 1);
    //Define os tipos de atributos a recuperar, Role e AccessId neste caso
    attrTypeRole := TAttributeType.Create(extFamily, Role);
    SetLength(attrTypeList, 2);
    attrTypeList[0] := attrTypeRole;
    attrTypeID      := TAttributeType.Create(extFamily, AccessId);
    attrTypeList[1] := attrTypeID;
    //Recupera os atributos de segurana das credenciais do usurio
    attrs := creds.get_attributes(attrTypeList);
    //Define o componente com o nome do recurso e a sua categoria - 'Aplicacoes'
    SetLength(resNameCompts, 1);
    resNameCompts[0] := TResourceNameComponent.Create('Aplicacoes', nomeRecurso);
    //Define o nome do recurso para o 'ado', composto pelo nome do servidor LDAP e
    //pela lista dos componentes do nome - 'resNameCompts'
    resNameCompt := TResourceName.Create('127.0.0.1', resNameCompts);
    try
      //Invoca o mtodo de solicitaao de autorizaao de acesso
      //Passa como parmetros o nome do recurso, a operao a realizar e os atributos
      //de segurana do usurio. Retorna um booleano indicando se o acesso est autorizado ou no
      result :=  ado.access_allowed(resNameCompt, operacao, attrs);
    except
      //O mtodo 'access_allowed' pode levantar esta exceao quando da ocorrncia de erros
      on exc : EInternalError do
        Writeln(exc.Message);
    end;
  end;
  //Libera memria dos arrays dinmicos
  resNameCompts   := nil;
  attrTypeList    := nil;
  attrs           := nil;
end;

Function mudaSenha(creds :Credentials; senhaAtual, novaSenha : String) : String;
const
  //Indica que o tipo de atributo  uma senha
  PRIVILEGE_ATTRIBUTE_CREDENTIALS = 10;
var
  extFamily    : ExtensibleFamily;
  attrType     : AttributeType;
  reqstedPriv  : SecAttribute;
  reqstedPrivList,
  attrsList    : Security_i.AttributeList;
  defAutho     : Opaque;
  pwdInfo      : Opaque;
  i            : Integer;
  n            : Integer;
  msg          : String;
begin
  result := 'credencial invlida';
  //Verifica se a credencial foi criada
  if (creds <> nil) then begin
    //Define a famlia dos atributos de segurana
    extFamily   := TExtensibleFamily.Create(0, 1);
    //Define o tipo de atributo a modificar, a senha neste caso
    attrType    := TAttributeType.Create(extFamily, PRIVILEGE_ATTRIBUTE_CREDENTIALS);
    //Define o valor do atributo, a senha atual e a nova senha separados pelo caracter de
    //'nova linha' - LINE FEED
    SetLength(pwdInfo, Length(senhaAtual) + Length(novaSenha) + 1);
    for i := 0 to Length(senhaAtual) - 1 do begin
      pwdInfo[i] := Byte(senhaAtual[i+1]);
    end;
    pwdInfo[Length(senhaAtual)] := Byte(#10);   //Line feed
    n := 1;
    for i := Length(senhaAtual) + 1 to Length(pwdInfo) - 1 do begin
      pwdInfo[i] := Byte(novaSenha[n]);
      Inc(n);
    end;
    reqstedPriv := TSecAttribute.Create(attrType, defAutho, pwdInfo);
    SetLength(reqstedPrivList, 1);
    reqstedPrivList[0] := reqstedPriv;
    SetLength(defAutho, 0);
    //Tenta modificar a senha chamando o mtodo set_privileges
    //O parmetro true indica que a mudana tem de ser imediata - FORCE_COMMIT = TRUE
    //'attrsList' retornar uma lista com um atributo cujo valor 
    //uma mensagem com o motivo do insucesso da modificao, caso ocorra
    //Retorna um booleano que indica se a senha foi modificada ou no
    if creds.set_privileges(true, reqstedPrivList, attrsList) then begin
      result :=  ''; //indica que a troca de senha foi bem sucedida
    end
    else begin
      if Length(attrsList) > 0 then begin   //Motivo para no trocar a senha
        SetString(msg, PChar(attrsList[0].value), Length(attrsList[0].value));
        result := msg;
      end;
    end;
  end;
  //Libera memria dos arrays dinmicos
  reqstedPrivList := nil;
  pwdInfo         := nil;
  attrsList       := nil;
  defAutho        := nil;
end;

Function pegaPapeisAtivos(creds : Credentials) : TStrings;
var
  extFamily       : ExtensibleFamily;
  attrType        : AttributeType;
  attrTypeList    : AttributeTypeList;
  attrs           : Security_i.AttributeList;
  userActiveRoles : TStrings;
  auxStr          : String;
  pos             : Integer;
begin
  userActiveRoles := TStringList.Create;
  //Verifica se a credencial foi criada
  if (creds <> nil) then begin
    //Verifica se a credencial foi criada
    extFamily := TExtensibleFamily.Create(0, 1);
    //Define o tipo de atributo a recuperar, os papis (Role) do usurio, neste caso
    attrType  := TAttributeType.Create(extFamily, Role);
    SetLength(attrTypeList, 1);
    attrTypeList[0] := attrType;
    //Recupera o valor do tipo de atributo a partir da credencial
    //O primeiro elemento de attrs tem a lista dos papis ativos separados por ";".  um string
    //vazio quando no h papis ativados
    //Os demais elementos so desprezados neste caso
    attrs := creds.get_attributes(attrTypeList);
    //Examina a lista de valores em busca do tipo de atributo requisitado
    //Ler apenas o primeiro elemento do vetor porque se procura pelos papis ativos
    if Length(attrs) > 0 then begin
      if (attrs[0].attribute_type.attribute_family.family_definer = 0) and
         (attrs[0].attribute_type.attribute_family.family = 1) and
         (attrs[0].attribute_type.attribute_type = Role) then begin
        SetString(auxStr, PChar(attrs[0].value), Length(attrs[0].value));
      end;
      //Separa a lista de papis ativados e coloca na lista 'userActiveRoles'
      pos := AnsiPos(';', auxStr);
      while (pos > 0) do begin
        userActiveRoles.Add(Copy(auxStr, 1, pos-1));
        Delete(auxStr, 1, pos);
        pos := AnsiPos(';', auxStr);
      end;
      if Length(auxStr) > 0 then
        userActiveRoles.Add(auxStr);
    end;
  end;
  result       := userActiveRoles;
  //Libera memria dos arrays dinmicos
  attrs        := nil;
  attrTypeList := nil;
end;

Function ativaPapel(creds : Credentials; umPapel : String) : Boolean;
var
  extFamily    : ExtensibleFamily;
  attrType     : AttributeType;
  reqstedPriv  : SecAttribute;
  reqstedPrivList,
  attrsList    : Security_i.AttributeList;
  defAutho     : Opaque;
  newActiveRole: Opaque;
  i            : Integer;
begin
  result := false;
  //Verifica se a credencial foi criada
  if (creds <> nil) then begin
    //Define a famlia dos atributos de segurana
    extFamily   := TExtensibleFamily.Create(0, 1);
    //Define o tipo de atributo a ativar, Role neste caso
    attrType    := TAttributeType.Create(extFamily, Role);
    //Define o valor do atributo, neste caso o papel passado como parmetro
    SetLength(newActiveRole, Length(umPapel));
    for i := 0 to Length(umPapel) - 1 do begin
      newActiveRole[i] := Byte(umPapel[i+1]);
    end;
    SetLength(defAutho, 0);
    //Cria a lista com os papis (atributos) para ativar (solicitados)
    reqstedPriv := TSecAttribute.Create(attrType, defAutho, newActiveRole);
    SetLength(reqstedPrivList, 1);
    reqstedPrivList[0] := reqstedPriv;
    //Tenta ativar o papel chamando o mtodo set_privileges
    //O parmetro true indica que a mudana tem de ser imediata - FORCE_COMMIT = TRUE
    //'attrsList' retorna a lista dos papis efetivamente ativados
    //Retorna um booleano que indica se o papel foi ativado ou no
    result :=  creds.set_privileges(true, reqstedPrivList, attrsList)
  end;
  //Libera memria dos arrays dinmicos
  reqstedPrivList := nil;
  newActiveRole   := nil;
  attrsList       := nil;
  defAutho        := nil;
end;

Function pegaPapeis (creds : Credentials) : TStrings;
var
  extFamily    : ExtensibleFamily;
  attrType     : AttributeType;
  attrTypeList : AttributeTypeList;
  attrs        : Security_i.AttributeList;
  n            : Integer;
  userRoles    : TStrings;
  auxStr       : String;
begin
  userRoles := TStringList.Create;
  //Verifica se a credencial foi criada
  if (creds <> nil) then begin
    //Define a famlia dos atributos de segurana
    extFamily := TExtensibleFamily.Create(0, 1);
    //Define o tipo de atributo a recuperar, os papis (Role) do usurio, neste caso
    attrType  := TAttributeType.Create(extFamily, Role);
    SetLength(attrTypeList, 1);
    attrTypeList[0] := attrType;
    //Recupera o valor do tipo de atributo a partir da credencial
    //O primeiro elemento de attrs tem a lista dos papis ativos separados por ";".  um string
    //vazio quando no h papis ativados
    //Os demais elementos do tipo Role correspondem aos papis associados ao usurio
    attrs := creds.get_attributes(attrTypeList);
    for n := 1 to Length(attrs)-1 do begin
      if (attrs[n].attribute_type.attribute_family.family_definer = 0) and
         (attrs[n].attribute_type.attribute_family.family = 1) and
         (attrs[n].attribute_type.attribute_type = Role) then begin
        SetString(auxStr, PChar(attrs[n].value), Length(attrs[n].value));
        userRoles.Add(auxStr);
      end;
    end;
  end;
  result       := userRoles;
  //Libera memria dos arrays dinmicos
  attrTypeList := nil;
  attrs        := nil;
end;

Function pegaAtributosUsuario (creds : Credentials) : TStrings;
var
  extFamily    : ExtensibleFamily;
  attrType     : AttributeType;
  attrTypeList : AttributeTypeList;
  attrs        : Security_i.AttributeList;
  n            : Integer;
  atributos    : TStrings;
  auxStr       : String;
begin
  atributos := TStringList.Create;
  //Verifica se a credencial foi criada
  if (creds <> nil) then begin
    //Define a famlia dos atributos de segurana
    extFamily := TExtensibleFamily.Create(0, 1);
    //Define o tipo de atributo a recuperar, o conjunto de atributos da conta do usurio, neste caso
    attrType  := TAttributeType.Create(extFamily, AttributeSet);
    SetLength(attrTypeList, 1);
    attrTypeList[0] := attrType;
    //Recupera o valor do tipo de atributo a partir da credencial
    //Cada elemento da lista retorna um atributo, do tipo string, com o seguinte formato:
    //<nome do atributo>: <valor_1 do atributo>, <valor_2 do atributo>, ..., <valor_n do atributo>
    attrs := creds.get_attributes(attrTypeList);
    //Examina a lista de valores em busca do tipo de atributo requisitado
    for n := 0 to Length(attrs)-1 do begin
      if (attrs[n].attribute_type.attribute_family.family_definer = 0) and
         (attrs[n].attribute_type.attribute_family.family = 1) and
         (attrs[n].attribute_type.attribute_type = AttributeSet) then begin
        SetString(auxStr, PChar(attrs[n].value), Length(attrs[n].value));
        atributos.Add(auxStr);
      end;
    end;
  end;
  result       := atributos;
  //Libera memria dos arrays dinmicos
  attrTypeList := nil;
  attrs        := nil;
end;

Function pegaLogin (creds : Credentials) : String;
var
  extFamily    : ExtensibleFamily;
  attrType     : AttributeType;
  attrTypeList : AttributeTypeList;
  attrs        : AttributeList;
  n            : Integer;
  login        : String;
begin
  result := '';
  //Verifica se a credencial foi criada
  if (creds <> nil) then begin
    //Define a famlia dos atributos de segurana
    //O primeiro argmento indica quem  o definidor da famlia
    //Valor 0 significa que o OMG   o definidor
    //O segundo argmento indica a famlia de atributos
    //Valor 1 significa que so atributos de privilgio
    extFamily := TExtensibleFamily.Create(0, 1);
    //Define o tipo de atributo a recuperar, o login (AccessId) do usurio, neste caso
    attrType  := TAttributeType.Create(extFamily, AccessId);
    SetLength(attrTypeList, 1);
    attrTypeList[0] := attrType;
    //Recupera o valor do tipo de atributo a partir da credencial
    attrs := creds.get_attributes(attrTypeList);
    //Examina a lista de valores em busca do tipo de atributo requisitado
    for n := 0 to Length(attrs) - 1 do begin
      if ((attrs[n].attribute_type.attribute_family.family_definer = 0) and
          (attrs[n].attribute_type.attribute_family.family = 1) and
          (attrs[n].attribute_type.attribute_type = AccessId)) then begin
        SetString(login, PChar(attrs[n].value), Length(attrs[n].value));
        result := login;
        break;
      end;
    end;
  end;
  //Libera memria dos arrays dinmicos
  attrs        := nil;
  attrTypeList := nil;
end;

Function conectaAccessDecisionObject(rootContext : NamingContextExt;
                                     const adoIOR : String) : AccessDecision;
var
  corbaObj : CORBAObject;
begin
  if rootContext = nil then begin //Servio de nomes indisponvel. Usa o IOR obtido via HTTP
    //Conecta-se com o objeto CORBA servidor do tipo AccessDecision
    corbaObj := Orb.StringToObject(adoIOR);
  end
  else begin //Conecta-se com o objeto CORBA servidor do tipo AccessDecision via servio de nomes
           //O nome do objeto CORBA servidor do tipo AccessDecision  "maca_cs.ctx/authorizers.ctx/ado.server"
      corbaObj := rootContext.resolve_str('maca_cs.ctx/authorizers.ctx/ado.server');
  end;
  //Faz o "type casting" do objeto do tipo CORBAObject para o tipo AccessDecision
  result := TAccessDecisionHelper.Narrow(corbaObj, true);
end;

Function conectaPrincipalAuthenticator(rootContext : NamingContextExt;
                                       const paIOR : String) : PrincipalAuthenticator;
var
  corbaObj : CORBAObject;
begin
  if rootContext = nil then begin //Servio de nomes indisponvel. Usa o IOR obtido via HTTP
    //Conecta-se com o objeto CORBA servidor do tipo PrincipalAuthenticator
    corbaObj := Orb.StringToObject(paIOR);
  end
  else begin //Conecta-se com o objeto CORBA servidor do tipo PrincipalAuthenticator via servio de nomes
             //O nome do objeto CORBA servidor do tipo PrincipalAuthenticator 
             //"maca_cs.ctx/authenticators.ctx/pa.server"
    corbaObj := rootContext.resolve_str('maca_cs.ctx/authenticators.ctx/pa.server');
  end;
  //Faz o "type casting" do objeto do tipo CORBAObject para o tipo PrincipalAuthenticator
  result := TPrincipalAuthenticatorHelper.Narrow(corbaObj, true);
end;

//Ler os IORs dos objetos do servio MACA CS
//Os IORs vm no seguinte formato:
//paIOR<identificado do objeto>;adoIOR<identificado do objeto>;
//Ou seja, vm separados por ; e devem ser desmembrados e o
//string que antecede o token 'IOR' deve ser removido.
procedure ReadIORs(const maca_cs_URL : String; var paIOR, adoIOR : String);
var
  iorServer    : TNMHTTP;
  line         : String;
  index        : Integer;
begin
  iorServer := TNMHTTP.Create(nil);
  try
    paIOR := '';
    adoIOR := '';
    iorServer.Get(maca_cs_URL+'/maca_cs.iors');
    line  := iorServer.Body;
    index := Pos(';', line);
    while (index > 0) do begin
     if (AnsiStartsText('paIOR', line)) then begin
      paIOR := Copy(line, Length('pa') + 1, index -  Length('pa') - 1);
     end
     else if (AnsiStartsText('adoIOR', line)) then begin
      adoIOR := Copy(line, Length('ado') + 1, index - Length('ado') - 1);
     end;
     Delete(line, 1, index);
     index := Pos(';', line);
    end;
  except
    on exc : ESockError do
      MessageDlg('Falha na comunicao: '+maca_cs_URL+'. '+
                 'Entre em contato com o suporte.', mtError, [mbOK], 0);
    on exc : Exception do
      MessageDlg('Falha lendo IORs: '+exc.Message+'. Entre em contato com o suporte.',
                 mtError, [mbOK], 0);
  else
      MessageDlg('Falha lendo IORs. Entre em contato com o suporte.',
                 mtError, [mbOK], 0);
  end;
  iorServer.Free;
end;


const
  CLEAR_TEXT_PWD = 0;

var
  rootContext  : NamingContextExt;
  pa           : PrincipalAuthenticator;
  ado          : AccessDecision;
  creds        : Credentials;
  authStatus   : AuthenticationStatus;
  senha        : String;
  pwd          : Opaque;
  attrs        : AttributeList;
  contData     : Opaque;
  authEspec    : Opaque;
  response_data: Opaque;
  i            : Integer;
  roles        : TStrings;
  atributos    : TStrings;
  defaultRole  : String;
  recursos,
  operacoes    : TStrings;
  results      : BooleanList;
  cbkSk        : TCallbackSkeleton;
  cbkCreds     : CallbackCredentials;
  msg          : String;
  paIOR,
  adoIOR       : String;
begin
  //Ler via HTTP os IORs do servios de autenticao e de
  //controle de acesso junto ao MACA CS, isto , no
  //servidor onde o MACA CS foi instalado.
  readIORs('http://127.0.0.1', paIOR, adoIOR);

  //Inicializa o ORB
  CorbaInitialize;

  //Conecta-se com o servio de nomes do CORBA - rootContext
  try
    rootContext := nil;
    rootContext := TNamingContextExtHelper.Narrow(Orb.ResolveInitialReferences('NameService'), True);
  except
    on Exception do
      Writeln('Servio de nomes no localizado. Verifique a propriedade de inicializao do ORB: -ORBInitRef');
  end;

  //Conecta-se com o objeto CORBA servidor de autentico "pa"
  pa := conectaPrincipalAuthenticator(rootContext, paIOR);

  //Definio dos parmetros do mtodo de autenticao do PrincipalAuthenticator
  //A lista de atributos de segurana apenas para autenticao pode ser vazia
  SetLength(attrs, 0);
  //Parametros de retorno para continuao da autenticao - no usados
  SetLength(contData, 0);
  SetLength(authEspec, 0);

  //Define senha como um array de bytes
  senha := 'maca';
  SetLength(pwd, Length(senha));
  for i := 1 to Length(senha) do begin
    pwd[i-1] := Byte(senha[i]);
  end;

  //Faz a autenticao
  authStatus := pa.authenticate(CLEAR_TEXT_PWD, //Mdoto de autenticao
                                'nome.sobrenome', //Nome de login do usurio
                                pwd,            //Senha
                                attrs,          //Atributos de segurana fornecidos
                                creds,          //Retorna a credencial criada para o
                                                //usurio em caso de sucesso
                                contData,       //Valor de retorno no usado
                                authEspec);     //Valor de retorno no usado

   //Verifica o status da autenticao
   case authStatus of
     SecAuthSuccess, SecAuthContinue : begin
        if authStatus = SecAuthContinue then begin
          //Este caso ocorre quando o usurio  obrigado a mudar a senha
          //para completar o processo de autenticao. Neste caso, a
          //credencial criada para esta sesso do usurio  vlida,
          //mas tem funcionalidade restrita.
          //Recupera mensagem sobre a autenticao
          SetString(msg, PChar(authEspec), Length(authEspec));
          Writeln(msg);
          //Nesta situao a credencial fica com sua funcionalidade restrita
          //enquanto a senha no  trocada. Aps a troca, a credencial fica
          //completamente operacional.
          //Muda a senha do usurio
          msg := mudaSenha(creds, 'caca', 'bola');
          if msg <> '' then begin //A senha no foi trocada
            Writeln('Senha no modificada: '+msg);
            //sesso  encerrada
            creds._destroy;
            Readln;
            Exit;
          end;
          Writeln('Senha modificada');
          //Continua o processo de autenticao
          SetLength(response_data, 0);
          if pa.continue_authentication(response_data, creds,
                                        contData, authEspec) <> SecAuthSuccess then begin
            Writeln('Falha na continuaao da autenticao!');
            //sesso  encerrada
            creds._destroy;
            Readln;
            Exit;
          end;
        end;
        Writeln('Autenticao realizada com sucesso!');
        //Recupera mensagem sobre a autenticao, por exemplo,
        //a que avisa que a senha ir expirar numa determinada data.
        //Recomenda-se sempre mostrar esta mensagem para o usurio
        SetString(msg, PChar(authEspec), Length(authEspec));
        Writeln(msg);

        //Encerra sesso do usurio destruindo a credencial
        creds._destroy;
        Writeln('Sesso encerrada!');
     end;
     SecAuthFailure : begin
       Writeln('Falha na autenticao!');
     end;
     SecAuthExpired : begin
       Writeln('Conta expirada!');
        //Recupera mensagem com informaes sobre a expirao
        //da conta do usurio
       SetString(msg, PChar(authEspec), Length(authEspec));
       Writeln(msg);
     end;
   end;


   //Finaliza o ORB no cliente
   orb.shutdown;
   readln;
   attrs     := nil;
   contData  := nil;
   authEspec := nil;
   pwd       := nil;
end.

