package tutorial;

import java.io.*;
import java.net.*;
import maca_cs.*;
import java.util.*;
import org.omg.CORBA.*;
import org.omg.CosNaming.*;
import org.omg.PortableServer.*;
import org.omg.maca_cs.Security.*;
import org.omg.maca_cs.SecurityLevel2.*;
import org.omg.maca_cs.DfResourceAccessDecision.*;
import org.omg.CosNaming.NamingContextExtPackage.*;

public class MacaClienteCompleto {

  //Mtodo de autenticao baseado em senha simples
  static final int CLEAR_TEXT_PWD = 0;

  static ORB orb;
  static NamingContextExt rootContext;
  static String  paIOR = "";
  static String adoIOR = "";

  public static void main(String[] args) {
    try {
      //Ler via HTTP os IORs do servios de autenticao e de
      //controle de acesso junto ao MACA CS, isto , no
      //servidor onde o MACA CS foi instalado
      readIORs("http://127.0.0.1");
      //Carrega as propriedades iniciais do ORB.
      FileInputStream st = new FileInputStream("./orb.properties");
      Properties orbProps = new Properties();
      orbProps.load(st);
      System.getProperties().putAll(orbProps);
      //Inicializa o ORB
      orb = ORB.init(args, System.getProperties());

      //Conecta-se com o servio de nomes do CORBA
      try {
        rootContext = NamingContextExtHelper.narrow(
                                         orb.resolve_initial_references("NameService"));
      }
      catch (Exception exc) {
        System.out.println("Servio de nomes no localizado. Verifique as "+
                           "propriedades do ORB:");
        System.out.println("     org.omg.CORBA.ORBInitialPort");
        System.out.println("     org.omg.CORBA.ORBInitialHost");
      }

      //Conecta-se com o objeto CORBA servidor de autentico "pa"
      PrincipalAuthenticator pa;
      pa = conectaPrincipalAuthenticator(rootContext);

      //Definio dos parmetros do mtodo de autenticao do PrincipalAuthenticator
      //A lista de atributos de segurana apenas para autenticao pode ser vazia
      SecAttribute[]         attrs = {};
      //Holder que retornar a credencial criada para o usurio
      CredentialsHolder credHolder = new CredentialsHolder();
      //Parametros de retorno para continuao da autenticao - no usados
      OpaqueHolder        contData = new OpaqueHolder();
      OpaqueHolder       authEspec = new OpaqueHolder();
      //Faz a autenticao
      AuthenticationStatus authStatus = pa.authenticate(CLEAR_TEXT_PWD,    //Mdoto de autenticao
                                                        "nome.sobrenome",  //Nome de login do usurio
                                                        "maca".getBytes(), //Senha
                                                        attrs,             //Atributos de segurana fornecidos
                                                        credHolder,        //Retorna a credencial criada para o
                                                                           //usurio em caso de sucesso
                                                        contData,          //Valor de retorno no usado
                                                        authEspec);        //Valor de retorno no usado
      //Verifica o status da autenticao
      switch (authStatus.value()) {
        case AuthenticationStatus._SecAuthContinue  : {
          //Este caso ocorre quando o usurio  obrigado a mudar a senha
          //para completar o processo de autenticao. Neste caso,
          // preciso recuperar credencial criada para esta sesso do usurio
          Credentials creds = credHolder.value;
          //Recupera mensagem sobre a autenticao
          String authMessage = new String(authEspec.value);
          System.out.println(authMessage);
          //Nesta situao a credencial fica com sua funcionalidade restrita
          //enquanto a senha no  trocada. Aps a troca, a credencial fica
          //completamente operacional.
          //Muda a senha do usurio
          String msg = mudaSenha(creds, "maca", "bola");
          if (!msg.equalsIgnoreCase("")) { //A senha no foi trocada
            System.out.println("Senha no modificada: "+msg);
            //sesso  encerrada
            creds.destroy();
            break;
          }
          System.out.println("Senha modificada");
          //Continua o processo de autenticao
          if (pa.continue_authentication(new byte[0], creds,
                                         contData, authEspec) != AuthenticationStatus.SecAuthSuccess) {
            System.out.println("Falha na continuaao da autenticao!");
            //sesso  encerrada
            creds.destroy();
            break;
          }
        }
        case AuthenticationStatus._SecAuthSuccess : {
          System.out.println("Autenticao realizada com sucesso!");
          //Recupera mensagem sobre a autenticao, por exemplo,
          //a que avisa que a senha ir expirar numa determinada data.
          //Recomenda-se sempre mostrar esta mensagem para o usurio
          String authMessage = new String(authEspec.value);
          System.out.println(authMessage);
          //Recupera credencial criada para esta sesso do usurio
          Credentials creds = credHolder.value;
          //Imprime o login do usurio
          System.out.println("Login: "+pegaLogin(creds));
          //Recupera o conjunto de atributos do usurio
          String[] atributos = pegaAtributosUsuario(creds);
          System.out.println("Atributos: ");
          for (int i = 0; i < atributos.length; i++) {
            System.out.println(atributos[i]);
          }
          //Recupera a lista de papis do usurio
          String[] roles = pegaPapeis(creds);
          System.out.print("Papis: ");
          for (int i = 0; i < roles.length; i++) {
            System.out.print(roles[i]+(i==(roles.length-1)?"\n":", "));
          }
          //Ativa o papel default
          if (roles.length > 0) {
            String defaultRole = roles[0];
            System.out.println("Papel '"+defaultRole+
                               (ativaPapel(creds, defaultRole)?"' ":"' no ")+
                               "ativado!");
          }
          //Recupera a lista de papis ativados para o usurio
          roles = pegaPapeisAtivos(creds);
          System.out.print("Papis ativos: ");
          for (int i = 0; i < roles.length; i++) {
            System.out.print(roles[i]+(i==(roles.length-1)?"\n":", "));
          }
          //Muda a senha do usurio
          String msg = mudaSenha(creds, "caca", "bola");
          System.out.println("Senha"+
                             (msg.equalsIgnoreCase("")?" ":" no ")+
                             "modificada: "+msg);
          //Conecta-se com o objeto CORBA servidor de autorizao de acesso "ado"
          AccessDecision ado;
          ado = conectaAccessDecisionObject(rootContext);
          //Verifica se o usurio tem acesso ao recurso 'SGP'
          if (acessoAutorizado(creds, ado, "SGP", "execuo")) {
            System.out.println("Acesso autorizado: SGP - [execuo]");
            //Verifica se o usurio tem acesso ao recurso 'ConcederDesconto;Itens;Pedidos;SGP'
            //Mostra como passar parmetros para uma autorizao parametrizada
            if (acessoAutorizado(creds, ado, "ConcederDesconto;Itens;Pedidos;SGP(35)", "execuo")) {
              System.out.println("Acesso autorizado: ConcederDesconto;Itens;Pedidos;SGP(35) - [execuo]");
            }
            else {
              System.out.println("Acesso NO autorizado: ConcederDesconto;Itens;Pedidos;SGP(35) - [execuo]");
            }
          }
          else {
            System.out.println("Acesso NO autorizado: SGP - [execuo]");
          }
          //Usa mtodo que faz mltiplas solicitaoes de acesso
          //Define array com dois nomes de recursos
          String[]  recursos = {"SGP", "Pedidos;SGP(1, 1, \"gustavo.motta\")"};
          //Define array com duas formas de acesso ou operaes, uma para cada recurso anterior
          String[] operacoes = {"execuo", "alterao"};
          boolean[]   result = autorizaMultiplosAcessos(creds, ado, recursos, operacoes);
          for (int n = 0; n < recursos.length; n++) {
            System.out.println("Acesso"+(result[n]?"":" NO")+" autorizado: "+recursos[n]+" - ["+operacoes[n]+"]");
          }

          //Exemplo de uso do callback

          //Primeiro,  preciso recuperar o acesso ao "RootPOA" do CORBA,
          //que vai habilitar a ativao do objeto de callback como um
          //servidor no cliente
          POA rootPOA = POAHelper.narrow(orb.resolve_initial_references("RootPOA"));
          //Depois, cria-se uma instncia da interface "Callback", ativando-a
          //atravs do POA. Assim, esta instncia passa a atuar como um
          //objeto servidor no cliente apto a receber requisies
          Callback callback = CallbackHelper.narrow(rootPOA.servant_to_reference(new CallbackImpl()));
          rootPOA.the_POAManager().activate();

          //Como a interface "Credentials" do CORBAsec no oferece facilidades
          //para definio no servidor do MACA CS de um objeto de callback, foi
          //definida a interface "CallbackCredentials", que estende a interface
          //"Credentials" com a incluso do mtodo "setClientCallback". Neste
          //caso,  preciso fazer o "type casting" do objeto creds com tipo
          //"Credentials" para o subtipo "CallbackCredentials" para se ter
          //acesso ao mtodo "setClientCallback".
          CallbackCredentials cbkCreds = CallbackCredentialsHelper.narrow(creds);
          //Define o objeto de callback do cliente
          //no servidor do MACA CS
          cbkCreds.setClientCallback(callback);

          System.out.println("Espera pelo callback do servidor MACA CS ...");
          System.out.println("    Tecle <enter> para encerrar sem testar ou ...");
          System.out.println("    aguarde pela mensagem de callback ou ...");
          System.out.println("    aborte o programa para verificar se o callback funciona");
          //Espera pelo callback
          System.in.read();

          //Encerra sesso do usurio destruindo a credencial
          creds.destroy();
          System.out.println("Sesso encerrada!");
          break;
        }
        case AuthenticationStatus._SecAuthFailure : {
          System.out.println("Falha na autenticao!");
          break;
        }
        case AuthenticationStatus._SecAuthExpired : {
          System.out.println("Senha expirada!");
          //Recupera mensagem com informaes sobre a expirao
          //da conta do usurio
          System.out.println(new String(authEspec.value));
          break;
        }
      }
      //Finaliza o ORB no cliente
      orb.shutdown(false);
    }
    catch (Exception exc) {
      exc.printStackTrace();
    }
  }

  static PrincipalAuthenticator conectaPrincipalAuthenticator(NamingContextExt rootContext) throws Exception {
    org.omg.CORBA.Object corbaObj;
    if (rootContext == null) { //Servio de nomes indisponvel. Usa o IOR obtido via HTTP
      //Conecta-se com o objeto CORBA servidor do tipo PrincipalAuthenticator
      corbaObj = orb.string_to_object(paIOR);
    }
    else { //Conecta-se com o objeto CORBA servidor do tipo PrincipalAuthenticator via servio de nomes
           //O nome do objeto CORBA servidor do tipo PrincipalAuthenticator 
           //"maca_cs.ctx/authenticators.ctx/pa.server"
      corbaObj = rootContext.resolve_str("maca_cs.ctx/authenticators.ctx/pa.server");
    }
    //Faz o "type casting" do objeto do tipo org.omg.CORBA.Object para o tipo PrincipalAuthenticator
    PrincipalAuthenticator pa = PrincipalAuthenticatorHelper.narrow(corbaObj);
    return pa;
  }

  static AccessDecision conectaAccessDecisionObject(NamingContextExt rootContext) throws Exception {
    org.omg.CORBA.Object corbaObj;
    if (rootContext == null) { //Servio de nomes indisponvel. Usa o IOR obtido via HTTP
      //Conecta-se com o objeto CORBA servidor do tipo AccessDecision
      corbaObj = orb.string_to_object(adoIOR);
    }
    else { //Conecta-se com o objeto CORBA servidor do tipo AccessDecision via servio de nomes
           //O nome do objeto CORBA servidor do tipo AccessDecision  "maca_cs.ctx/authorizers.ctx/ado.server"
      corbaObj = rootContext.resolve_str("maca_cs.ctx/authorizers.ctx/ado.server");
    }
    //Faz o "type casting" do objeto do tipo org.omg.CORBA.Object para o tipo AccessDecision
    AccessDecision ado = AccessDecisionHelper.narrow(corbaObj);
    return ado;
  }

  static String pegaLogin (Credentials creds) {
    //Verifica se a credencial foi criada
    if (creds != null) {
      //Define a famlia dos atributos de segurana
      //O primeiro argmento indica quem  o definidor da famlia
      //Valor 0 significa que o OMG   o definidor
      //O segundo argmento indica a famlia de atributos
      //Valor 1 significa que so atributos de privilgio
      ExtensibleFamily extFamily = new ExtensibleFamily((short) 0, (short) 1);
      //Define o tipo de atributo a recuperar, o login (AccessId) do usurio, neste caso
      AttributeType[]   attrType = {new AttributeType(extFamily, AccessId.value)};
      //Recupera o valor do tipo de atributo a partir da credencial
      SecAttribute[]       attrs = creds.get_attributes(attrType);
      //Examina a lista de valores em busca do tipo de atributo requisitado
      for (int n = 0; n < attrs.length; n++) {
        if ((attrs[n].attribute_type.attribute_family.family_definer == 0) &&
            (attrs[n].attribute_type.attribute_family.family == 1) &&
            (attrs[n].attribute_type.attribute_type == AccessId.value)) {
          return new String(attrs[n].value);
        }
      }
    }
    return "";
  }

  static String[] pegaPapeis (Credentials creds) {
    String[] roles = {};
    //Verifica se a credencial foi criada
    if (creds != null) {
      //Define a famlia dos atributos de segurana
      ExtensibleFamily extFamily = new ExtensibleFamily((short) 0, (short) 1);
      //Define o tipo de atributo a recuperar, os papis (Role) do usurio, neste caso
      AttributeType[]   attrType = {new AttributeType(extFamily, Role.value)};
      //Recupera o valor do tipo de atributo a partir da credencial
      //O primeiro elemento de attrs tem a lista dos papis ativos separados por ";".  um string
      //vazio quando no h papis ativados
      //Os demais elementos do tipo Role correspondem aos papis associados ao usurio
      SecAttribute[]       attrs = creds.get_attributes(attrType);
      //Define a quantidade de papis associados retornados
      roles = new String[attrs.length > 0?attrs.length-1:0];
      //Examina a lista de valores em busca do tipo de atributo requisitado
      //Comea pelo segundo elemento do vetor porque se procura por papis associados e no os ativos
      for (int n = 1; n < attrs.length; n++) {
        if ((attrs[n].attribute_type.attribute_family.family_definer == 0) &&
            (attrs[n].attribute_type.attribute_family.family == 1) &&
            (attrs[n].attribute_type.attribute_type == Role.value)) {
          roles[n-1] = new String(attrs[n].value);
        }
      }
    }
    return roles;
  }


  static String[] pegaAtributosUsuario (Credentials creds) {
    String[] atributos = {};
    //Verifica se a credencial foi criada
    if (creds != null) {
      //Define a famlia dos atributos de segurana
      ExtensibleFamily extFamily = new ExtensibleFamily((short) 0, (short) 1);
      //Define o tipo de atributo a recuperar, o conjunto de atributos da conta do usurio, neste caso
      AttributeType[]   attrType = {new AttributeType(extFamily, AttributeSet.value)};
      //Recupera o valor do tipo de atributo a partir da credencial
      //Cada elemento da lista retorna um atributo, do tipo string, com o seguinte formato:
      //<nome do atributo>: <valor_1 do atributo>, <valor_2 do atributo>, ..., <valor_n do atributo>
      SecAttribute[]       attrs = creds.get_attributes(attrType);
      //Define a quantidade de atributos da conta retornados
      atributos = new String[attrs.length];
      //Examina a lista de valores em busca do tipo de atributo requisitado
      for (int n = 0; n < attrs.length; n++) {
        if ((attrs[n].attribute_type.attribute_family.family_definer == 0) &&
            (attrs[n].attribute_type.attribute_family.family == 1) &&
            (attrs[n].attribute_type.attribute_type == AttributeSet.value)) {
          atributos[n] = new String(attrs[n].value);
        }
      }
    }
    return atributos;
  }

  static boolean ativaPapel(Credentials creds, String umPapel) {
    //Verifica se a credencial foi criada
    if (creds != null) {
      //Define a famlia dos atributos de segurana
      ExtensibleFamily extFamily = new ExtensibleFamily((short) 0, (short) 1);
      //Define o tipo de atributo a ativar, Role neste caso
      AttributeType    attrType  = new AttributeType(extFamily, Role.value);
      //Define o valor do atributo, neste caso o papel passado como parmetro
      SecAttribute[] reqstedPriv = {new SecAttribute(attrType, new byte [0], umPapel.getBytes())};
      //Define um Holder que retornar a lista dos atributos efetivamnete ativados
      org.omg.maca_cs.Security.AttributeListHolder actualPriv = new org.omg.maca_cs.Security.AttributeListHolder();
      //Tenta ativar o papel chamando o mtodo set_privileges
      //O parmetro true indica que a mudana tem de ser imediata - FORCE_COMMIT = TRUE
      //Retorna um booleano que indica se o papel foi ativado ou no
      return creds.set_privileges(true, reqstedPriv, actualPriv);
    }
    return false;
  }

  static String[] pegaPapeisAtivos (Credentials creds) {
    String[] roles = {};
    //Verifica se a credencial foi criada
    if (creds != null) {
      //Define a famlia dos atributos de segurana
      ExtensibleFamily extFamily = new ExtensibleFamily((short) 0, (short) 1);
      //Define o tipo de atributo a recuperar, os papis (Role) do usurio, neste caso
      AttributeType[]   attrType = {new AttributeType(extFamily, Role.value)};
      //Recupera o valor do tipo de atributo a partir da credencial
      //O primeiro elemento de attrs tem a lista dos papis ativos separados por ";".  um string
      //vazio quando no h papis ativados
      //Os demais elementos so desprezados neste caso
      SecAttribute[]       attrs = creds.get_attributes(attrType);
      String activeRoles = "";
      //Examina a lista de valores em busca do tipo de atributo requisitado
      //Ler apenas o primeiro elemento do vetor porque se procura pelos papis ativos
      for (int n = 0; n < attrs.length; n++) {
        if ((attrs[n].attribute_type.attribute_family.family_definer == 0) &&
            (attrs[n].attribute_type.attribute_family.family == 1) &&
            (attrs[n].attribute_type.attribute_type == Role.value)) {
          activeRoles = new String(attrs[n].value);
          break;
        }
      }
      //Separa a lista de papis ativados e coloca o array 'roles'
      StringTokenizer strTk = new StringTokenizer(activeRoles, ";");
      roles = new String[strTk.countTokens()];
      for (int n = 0; strTk.hasMoreTokens(); n++) {
        roles[n] = strTk.nextToken();
      }
    }
    return roles;
  }

  static String mudaSenha(Credentials creds, String senhaAtual, String novaSenha) {
    //Indica que o tipo de atributo  uma senha
    final int PRIVILEGE_ATTRIBUTE_CREDENTIALS = 10;
    //Verifica se a credencial foi criada
    if (creds != null) {
      //Define a famlia dos atributos de segurana
      ExtensibleFamily extFamily = new ExtensibleFamily((short) 0, (short) 1);
      //Define o tipo de atributo a modificar, a senha neste caso
      AttributeType    attrType  = new AttributeType(extFamily, PRIVILEGE_ATTRIBUTE_CREDENTIALS);
      //Define o valor do atributo, a senha atual e a nova senha separados pelo caracter de
      //'nova linha' - LINE FEED
      SecAttribute[] reqstedPriv = {new SecAttribute(attrType, new byte [0],
                                                     (senhaAtual+"\n"+novaSenha).getBytes())};
      //Define um Holder que retornar a lista com um atributo cujo valor 
      //uma mensagem com o motivo do insucesso da modificao
      org.omg.maca_cs.Security.AttributeListHolder actualPriv = new org.omg.maca_cs.Security.AttributeListHolder();
      //Tenta modificar a senha chamando o mtodo set_privileges
      //O parmetro true indica que a mudana tem de ser imediata - FORCE_COMMIT = TRUE
      //Retorna um booleano que indica se a senha foi modificada ou no
      boolean changed = creds.set_privileges(true, reqstedPriv, actualPriv);
      String  message = "";
      if (!changed && (actualPriv.value.length > 0)) {
        //Motivo para no trocar a senha
        message = new String(actualPriv.value[0].value);
      }
      return message;
    }
    return "credenciais invlidas";
  }

  static boolean acessoAutorizado (Credentials creds, AccessDecision ado,
                                   String nomeRecurso, String operacao) {
    //Verifica se a credencial e 'ado' foram criados
    if ((creds != null) && (ado != null)) {
      //Define a famlia dos atributos de segurana
      ExtensibleFamily extFamily = new ExtensibleFamily((short) 0, (short) 1);
      //Define os tipos de atributos a recuperar, Role e AccessId neste caso
      AttributeType[] attrTypes = {new AttributeType(extFamily, Role.value),
                                  new AttributeType(extFamily, AccessId.value)};
      //Recupera os atributos de segurana das credenciais do usurio
      SecAttribute[] attrs = creds.get_attributes(attrTypes);
      //Define o componente com o nome do recurso e a sua categoria - 'Aplicacoes'
      ResourceNameComponent[] resNameCompts = {new ResourceNameComponent("Aplicacoes", nomeRecurso)};
      //Define o nome do recurso para o 'ado', composto pelo nome do servidor LDAP e pela lista
      //dos componentes do nome
      ResourceName resName = new ResourceName("127.0.0.1", resNameCompts);
      try {
        //Invoca o mtodo de solicitaao de autorizaao de acesso
        //Passa como parmetros o nome do recurso, a operao a realizar e os atributos
        //de segurana do usurio. Retorna um booleano indicando se o acesso est autorizado ou no
        return ado.access_allowed(resName, operacao, attrs);
      }
      //O mtodo 'access_allowed' pode levantar esta exceao quando da ocorrncia de erros
      catch (org.omg.maca_cs.DfResourceAccessDecision.InternalError exc) {
        System.out.println(exc);
      }
    }
    return false;
  }

  static boolean[] autorizaMultiplosAcessos (Credentials creds, AccessDecision ado,
                                           String[] nomeRecursos, String[] operacoes) {
    //Verifica se a credencial e 'ado' foram criados
    if ((creds != null) && (ado != null) && (nomeRecursos.length == operacoes.length)) {
      try {
        //Define a famlia dos atributos de segurana
        ExtensibleFamily extFamily = new ExtensibleFamily((short) 0, (short) 1);
        //Define os tipos de atributos a recuperar, Role e AccessId neste caso
        AttributeType[] attrTypes = {new AttributeType(extFamily, Role.value),
                                    new AttributeType(extFamily, AccessId.value)};
        //Recupera os atributos de segurana das credenciais do usurio
        SecAttribute[] attrs      = creds.get_attributes(attrTypes);
        //Define a lista de requisies de autorizao de acesso
        AccessDefinition[] access_requests = new AccessDefinition[nomeRecursos.length];
        //Para cada recurso, monta o componente com o nome do recurso e operao, e o inclui
        //na lista 'access_requests'
        for (int i = 0; i < access_requests.length; i++) {
          ResourceNameComponent[] resNameCompts =
                              {new ResourceNameComponent("Aplicacoes", nomeRecursos[i])};
          ResourceName resName = new ResourceName("127.0.0.1", resNameCompts);
          access_requests[i]   = new AccessDefinition(resName, operacoes[i]);
        };
        //Invoca o mtodo que retorna um array de booleanos, cada booleano correspondendo ao
        //resultado da solicitaao de acesso ao recurso correspondente a mesma posio do array
        return ado.multiple_access_allowed(access_requests, attrs);
      }
      catch (org.omg.maca_cs.DfResourceAccessDecision.InternalError exc) {
        System.out.println(exc);
      }
    }
    return new boolean[0];
  }


  //Ler os IORs dos objetos do servio MACA CS
  //Os IORs vm no seguinte formato:
  //paIOR<identificado do objeto>;adoIOR<identificado do objeto>;
  //Ou seja, vm separados por ; e devem ser desmembrados e o
  //string que antecede o token 'IOR' deve ser removido.
  private static void readIORs(String maca_cs_URL) throws Exception {
    URL iorURL = new URL(maca_cs_URL+"/maca_cs.iors");
    BufferedReader iorStream = new BufferedReader(new InputStreamReader(iorURL.openStream()));
    String line = iorStream.readLine();
    StringTokenizer strTk = new StringTokenizer(line, ";");
    while (strTk.hasMoreTokens()) {
      String ior = strTk.nextToken();
      if (ior.startsWith("paIOR")) {
        paIOR = ior.substring("pa".length());
      }
      else if (ior.startsWith("adoIOR")) {
        adoIOR = ior.substring("ado".length());
      }
    }
    iorStream.close();
  }



  //Classe que implementa o mtodo de callback "isClientAlive"
  //no cliente. Este  o nico mtodo existente na interface
  //"Callback" e deve ser implementado com a extenso da
  //classe "CallbackPOA"
  static class CallbackImpl extends CallbackPOA {
    CallbackImpl() {
    }

    //Mtodo de callback implementado pelo cliente e
    //invocado pelo servidor MACA CS. Embora seja chamado
    //com modo "oneway", sua execuo deve ser a mais breve
    //possvel
    public void isClientAlive (String message) {
      System.out.println(message);
    }
  }

}